use actix_web::{
    error::ErrorInternalServerError,
    web::{self, Path},
    HttpResponse, Responder,
};
use sea_orm::{ActiveModelTrait, ActiveValue, EntityTrait, IntoActiveModel};
use serde::{Deserialize, Serialize};
use uuid::Uuid;

use crate::{auth::AuthedUser, AppState};

use super::user::UserWithoutPassword;

#[derive(Serialize)]
struct GroupWithLicenses {
    #[serde(flatten)]
    group: entity::license_group::Model,
    licenses: Vec<entity::license::Model>,
}

pub struct LicenseController;

impl LicenseController {
    pub async fn list_groups(
        state: web::Data<AppState>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let groups = entity::prelude::LicenseGroup::find()
            .find_with_related(entity::prelude::License)
            .all(db)
            .await
            .map_err(ErrorInternalServerError)?
            .into_iter()
            .map(|(group, licenses)| GroupWithLicenses { group, licenses })
            .collect::<Vec<_>>();
        Ok(web::Json(groups))
    }

    pub async fn create_group(
        state: web::Data<AppState>,
        group: web::Json<entity::license_group::Model>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let group = group.into_inner();
        let mut group = group.into_active_model();
        group.id = ActiveValue::NotSet;

        let res = group.insert(db).await.map_err(ErrorInternalServerError)?;

        Ok(web::Json(res))
    }
    pub async fn create_license(
        state: web::Data<AppState>,
        license: web::Json<entity::license::Model>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let license = license.into_inner();
        let mut license = license.into_active_model();
        license.id = ActiveValue::NotSet;

        let res = license.insert(db).await.map_err(ErrorInternalServerError)?;

        Ok(web::Json(res))
    }

    pub async fn update_license(
        state: web::Data<AppState>,
        license: web::Json<entity::license::Model>,
        path: Path<Uuid>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let license = license.into_inner();
        let id = path.into_inner();
        let license = entity::license::ActiveModel {
            id: ActiveValue::Unchanged(id),
            amount: ActiveValue::Set(license.amount),
            name: ActiveValue::Set(license.name),
            start: ActiveValue::Set(license.start),
            end: ActiveValue::Set(license.end),
            key: ActiveValue::Set(license.key),
            group_id: ActiveValue::Set(license.group_id),
            note: ActiveValue::Set(license.note),
        };
        let res = license.update(db).await.map_err(ErrorInternalServerError)?;
        Ok(web::Json(res))
    }

    pub async fn delete_license(
        state: web::Data<AppState>,
        path: Path<Uuid>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        entity::license::Entity::delete_by_id(path.into_inner())
            .exec(db)
            .await
            .map_err(ErrorInternalServerError)?;
        Ok(HttpResponse::Ok().finish())
    }

    pub async fn delete_group(
        state: web::Data<AppState>,
        path: Path<Uuid>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        entity::license_group::Entity::delete_by_id(path.into_inner())
            .exec(db)
            .await
            .map_err(ErrorInternalServerError)?;
        Ok(HttpResponse::Ok().finish())
    }
}
